library benchmarks.src.largetable.largetable_benchmark;

import "package:angular2/src/platform/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/facade/browser.dart" show window, document, gc;
import "package:angular2/src/testing/benchmark_util.dart"
    show
        getIntParameter,
        getStringParameter,
        bindAction,
        windowProfile,
        windowProfileEnd;
import "package:angular2/platform/browser.dart" show bootstrap;
import "package:angular2/core.dart" show Component, Directive, bind, provide;
import "package:angular2/common.dart"
    show NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault;
import "package:angular2/src/core/application_ref.dart" show ApplicationRef;
import "package:angular2/src/platform/browser/browser_adapter.dart"
    show BrowserDomAdapter;
import "package:angular2/src/facade/collection.dart" show ListWrapper;
import "package:angular2/src/core/di/decorators.dart" show Inject;
import "package:angular2/src/core/reflection/reflection.dart" show reflector;

const BENCHMARK_TYPE = "LargetableComponent.benchmarkType";
const LARGETABLE_ROWS = "LargetableComponent.rows";
const LARGETABLE_COLS = "LargetableComponent.cols";
_createBindings() {
  return [
    provide(BENCHMARK_TYPE, useValue: getStringParameter("benchmarkType")),
    provide(LARGETABLE_ROWS, useValue: getIntParameter("rows")),
    provide(LARGETABLE_COLS, useValue: getIntParameter("columns"))
  ];
}

var BASELINE_LARGETABLE_TEMPLATE;
setupReflector() {
  // TODO(kegluneq): Generate these.
  reflector.registerGetters({
    "benchmarktype": (o) => o.benchmarktype,
    "switch": (o) => null,
    "switchWhen": (o) => o.switchWhen
  });
  reflector.registerSetters({
    "benchmarktype": (o, v) => o.benchmarktype = v,
    "switch": (o, v) => null,
    "switchWhen": (o, v) => o.switchWhen = v
  });
}

main() {
  BrowserDomAdapter.makeCurrent();
  var totalRows = getIntParameter("rows");
  var totalColumns = getIntParameter("columns");
  BASELINE_LARGETABLE_TEMPLATE = DOM.createTemplate("<table></table>");
  var app;
  var appRef;
  var baselineRootLargetableComponent;
  ng2DestroyDom() {
    // TODO: We need an initial value as otherwise the getter for data.value will fail

    // --> this should be already caught in change detection!
    app.data = null;
    app.benchmarkType = "none";
    appRef.tick();
  }
  profile(create, destroy, name) {
    return () {
      windowProfile(name + " w GC");
      var duration = 0;
      var count = 0;
      while (count++ < 150) {
        gc();
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      windowProfileEnd(name + " w GC");
      window.console.log(
          '''Iterations: ${ count}; time: ${ duration / count} ms / iteration''');
      windowProfile(name + " w/o GC");
      duration = 0;
      count = 0;
      while (count++ < 150) {
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      windowProfileEnd(name + " w/o GC");
      window.console.log(
          '''Iterations: ${ count}; time: ${ duration / count} ms / iteration''');
    };
  }
  ng2CreateDom() {
    var data = ListWrapper.createFixedSize(totalRows);
    for (var i = 0; i < totalRows; i++) {
      data[i] = ListWrapper.createFixedSize(totalColumns);
      for (var j = 0; j < totalColumns; j++) {
        data[i][j] = new CellData(i, j);
      }
    }
    app.data = data;
    app.benchmarkType = getStringParameter("benchmarkType");
    appRef.tick();
  }
  noop() {}
  initNg2() {
    bootstrap(AppComponent, _createBindings()).then((ref) {
      var injector = ref.injector;
      app = ref.hostComponent;
      appRef = injector.get(ApplicationRef);
      bindAction("#ng2DestroyDom", ng2DestroyDom);
      bindAction("#ng2CreateDom", ng2CreateDom);
      bindAction(
          "#ng2UpdateDomProfile", profile(ng2CreateDom, noop, "ng2-update"));
      bindAction("#ng2CreateDomProfile",
          profile(ng2CreateDom, ng2DestroyDom, "ng2-create"));
    });
    setupReflector();
  }
  baselineDestroyDom() {
    baselineRootLargetableComponent.update(buildTable(0, 0));
  }
  baselineCreateDom() {
    baselineRootLargetableComponent.update(buildTable(totalRows, totalColumns));
  }
  initBaseline() {
    baselineRootLargetableComponent = new BaseLineLargetableComponent(
        DOM.querySelector(document, "baseline"),
        getStringParameter("benchmarkType"),
        getIntParameter("rows"),
        getIntParameter("columns"));
    bindAction("#baselineDestroyDom", baselineDestroyDom);
    bindAction("#baselineCreateDom", baselineCreateDom);
    bindAction("#baselineUpdateDomProfile",
        profile(baselineCreateDom, noop, "baseline-update"));
    bindAction("#baselineCreateDomProfile",
        profile(baselineCreateDom, baselineDestroyDom, "baseline-create"));
  }
  initNg2();
  initBaseline();
}

buildTable(rows, columns) {
  var tbody = DOM.createElement("tbody");
  var template = DOM.createElement("span");
  var i, j, row, cell;
  DOM.appendChild(template, DOM.createElement("span"));
  DOM.appendChild(template, DOM.createTextNode(":"));
  DOM.appendChild(template, DOM.createElement("span"));
  DOM.appendChild(template, DOM.createTextNode("|"));
  for (i = 0; i < rows; i++) {
    row = DOM.createElement("div");
    DOM.appendChild(tbody, row);
    for (j = 0; j < columns; j++) {
      cell = DOM.clone(template);
      DOM.appendChild(row, cell);
      DOM.setText(cell.childNodes[0], i.toString());
      DOM.setText(cell.childNodes[2], j.toString());
    }
  }
  return tbody;
}

class BaseLineLargetableComponent {
  var element;
  var table;
  String benchmarkType;
  num rows;
  num columns;
  BaseLineLargetableComponent(element, benchmarkType, num rows, num columns) {
    this.element = element;
    this.benchmarkType = benchmarkType;
    this.rows = rows;
    this.columns = columns;
    this.table = DOM.clone(BASELINE_LARGETABLE_TEMPLATE.content.firstChild);
    var shadowRoot = DOM.createShadowRoot(this.element);
    DOM.appendChild(shadowRoot, this.table);
  }
  update(tbody) {
    var oldBody = DOM.querySelector(this.table, "tbody");
    if (oldBody != null) {
      DOM.replaceChild(this.table, tbody, oldBody);
    } else {
      DOM.appendChild(this.table, tbody);
    }
  }
}

class CellData {
  num i;
  num j;
  CellData(i, j) {
    this.i = i;
    this.j = j;
  }
  jFn() {
    return this.j;
  }

  iFn() {
    return this.i;
  }
}

@Component(
    selector: "largetable",
    inputs: const ["data", "benchmarkType"],
    directives: const [NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault],
    template: '''
      <table [ngSwitch]="benchmarkType">
        <tbody template="ngSwitchWhen \'interpolation\'">
          <tr template="ngFor #row of data">
            <td template="ngFor #column of row">
              {{column.i}}:{{column.j}}|
            </td>
          </tr>
        </tbody>
        <tbody template="ngSwitchWhen \'interpolationAttr\'">
          <tr template="ngFor #row of data">
            <td template="ngFor #column of row" attr.i="{{column.i}}" attr.j="{{column.j}}">
              i,j attrs
            </td>
          </tr>
        </tbody>
        <tbody template="ngSwitchWhen \'interpolationFn\'">
          <tr template="ngFor #row of data">
            <td template="ngFor #column of row">
              {{column.iFn()}}:{{column.jFn()}}|
            </td>
          </tr>
        </tbody>
        <tbody template="ngSwitchDefault">
          <tr>
            <td>
              <em>{{benchmarkType}} not yet implemented</em>
            </td>
          </tr>
        </tbody>
      </table>''')
class LargetableComponent {
  var data;
  String benchmarkType;
  num rows;
  num columns;
  LargetableComponent(@Inject(BENCHMARK_TYPE) benchmarkType,
      @Inject(LARGETABLE_ROWS) rows, @Inject(LARGETABLE_COLS) columns) {
    this.benchmarkType = benchmarkType;
    this.rows = rows;
    this.columns = columns;
  }
}

@Component(
    selector: "app",
    directives: const [LargetableComponent],
    template:
        '''<largetable [data]=\'data\' [benchmarkType]=\'benchmarkType\'></largetable>''')
class AppComponent {
  var data;
  String benchmarkType;
}
